/*
 * $Header: /home/cvs/jakarta-slide/src/webdav/client/src/org/apache/commons/httpclient/HttpException.java,v 1.1 2001/08/10 20:39:50 remm Exp $
 * $Revision: 1.1 $
 * $Date: 2001/08/10 20:39:50 $
 *
 * ====================================================================
 *
 * The Apache Software License, Version 1.1
 *
 * Copyright (c) 1999 The Apache Software Foundation.  All rights
 * reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 *
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 *
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in
 *    the documentation and/or other materials provided with the
 *    distribution.
 *
 * 3. The end-user documentation included with the redistribution, if
 *    any, must include the following acknowlegement:
 *       "This product includes software developed by the
 *        Apache Software Foundation (http://www.apache.org/)."
 *    Alternately, this acknowlegement may appear in the software itself,
 *    if and wherever such third-party acknowlegements normally appear.
 *
 * 4. The names "The Jakarta Project", "Tomcat", and "Apache Software
 *    Foundation" must not be used to endorse or promote products derived
 *    from this software without prior written permission. For written
 *    permission, please contact apache@apache.org.
 *
 * 5. Products derived from this software may not be called "Apache"
 *    nor may "Apache" appear in their names without prior written
 *    permission of the Apache Group.
 *
 * THIS SOFTWARE IS PROVIDED ``AS IS'' AND ANY EXPRESSED OR IMPLIED
 * WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
 * OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
 * DISCLAIMED.  IN NO EVENT SHALL THE APACHE SOFTWARE FOUNDATION OR
 * ITS CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 * SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
 * LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF
 * USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
 * ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY,
 * OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT
 * OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
 * SUCH DAMAGE.
 * ====================================================================
 *
 * This software consists of voluntary contributions made by many
 * individuals on behalf of the Apache Software Foundation.  For more
 * information on the Apache Software Foundation, please see
 * <http://www.apache.org/>.
 *
 * [Additional notices, if required by prior licensing conditions]
 *
 */

package org.apache.commons.httpclient;

/**
 * Http exception class.
 */
public class HttpException
    extends Exception {


    // ----------------------------------------------------- Instance Variables


    /**
     * The status code.
     */
    protected int statusCode;


    /**
     * The message.
     */
    protected String message = "";


    // -------------------------------------------------------------- Constants


    /**
     * Method not processed.
     */
    public static final int METHOD_NOT_PROCESSED = -1;


    /**
     * No credential given.
     */
    public static final int NO_CREDENTIALS_GIVEN = -2;


    // ----------------------------------------------------------- Constructors


    /**
     * Constructor.
     *
     * @param message Exception message
     */
    public HttpException(String message) {
        this.message = message;
    }


    /**
     * Constructor.
     *
     * @param statuscode Exception statusCode
     */
    public HttpException(int statusCode) {
        this.statusCode = statusCode;
        if (statusCode < 0) {
            switch (statusCode) {
            case HttpException.METHOD_NOT_PROCESSED:
                message = "Method not processed";
                break;
            case HttpException.NO_CREDENTIALS_GIVEN:
                message = "No credentials given";
                break;
            default:
                break;
            }
        } else {
            String statusText = HttpStatus.getStatusText(statusCode);
            if (statusText != null) {
                message = statusText;
            }
        }
    }


    /**
     * Constructor.
     *
     * @param message Exception message
     * @param statuscode Exception statusCode
     */
    public HttpException(String message, int statusCode) {
        this(message);
        this.statusCode = statusCode;
    }


    // ------------------------------------------------------------- Properties


    /**
     * getStatusCode
     */
    public int getStatusCode() {
        return statusCode;
    }


    /**
     * getMessage
     */
    public String getMessage() {
        return message;
    }


}
